(* File: dumper.ml

   Copyright (C) 2011-2015

     Fabian Pijcke <fabian.pijcke@gmail.com>
     Pierre Hauweele <antegallya@gmail.com>

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License version 3 or
   later as published by the Free Software Foundation, with the special
   exception on linking described in the file LICENSE.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file
   LICENSE for more details. *)

open Common
open Printf

module W = Wythoff
module P = Point

let concat ?(sep=" ") = function
  | [] -> ""
  | hd :: tl -> List.fold_left (fun acc x -> acc ^ sep ^ x) hd tl

type show =
  | Faces
  | Lines

let to_vrml ?(number=false) ?(show=Faces) p =
  let write s = printf (s ^^ "\n") in
  (* File header *)
  write "#VRML V2.0 utf8\n";
  (* Background *)
  write "Background {";
  write "\tskyColor 0.0565629 0.2208154 0.4";
  write "}";
  (* Polyhedron header *)
  write "Shape {";
  write "\tappearance Appearance { material Material { } }";
  (match show with
  | Faces ->
    write "\tgeometry IndexedFaceSet {";
    write "\t\tsolid FALSE";
    write "\t\tcreaseAngle 4.000000"
  | Lines -> write "\tgeometry IndexedLineSet {");
  (* Point coordinates *)
  write "\t\tcoord Coordinate {point [";
  let coords = Array.map (function
    | P.Cartesian (x, y, z) -> sprintf "%f %f %f" x y z
    | P.Spherical _ -> assert false) (Array.map P.to_cartesian p.W.vertices)
  in
  let svertices = concat ~sep:",\n\t\t\t" (Array.to_list coords) in
  write "\t\t\t%s]}" svertices;
  (* Faces *)
  write "\t\tcoordIndex [";
  let faces = List.map (List.map string_of_int) p.W.faces in
  let closedfaces = List.map (fun l -> l @ [List.hd l]) faces in
  let sfaces = concat ~sep:" -1,\n\t\t\t" (List.map concat closedfaces) in
  write "\t\t\t%s -1]" sfaces;
  (* Polyhedron footer *)
  write "\t}";
  write "}";
  (* Number vertices *)
  if number then
    Array.iteri (fun i textcoord ->
      write "Transform {";
      write "\ttranslation %s" textcoord;
      write "\tchildren [ Shape { geometry Text {";
      write "\t\tstring \"%i\"" i;
      write "\t\tfontStyle FontStyle { size 0.042 }";
      write "} } ]";
      write "}") coords;
  (* Viewpoint *)
  write "DEF Viewpoint1 Viewpoint {";
  write "\torientation 0 1 0 3.141592";
  write "\tposition 0 0 -5";
  write "\tdescription \"Viewpoint1\"";
  write "}"
