(* File: display.ml

   Copyright (C) 2011-2015

     Fabian Pijcke <fabian.pijcke@gmail.com>
     Pierre Hauweele <antegallya@gmail.com>

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License version 3 or
   later as published by the Free Software Foundation, with the special
   exception on linking described in the file LICENSE.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file
   LICENSE for more details. *)

open Printf
open Wythoff

module P = Point

let () =
  let parse s =
    let pn, pd, qn, qd, rn, rd, bp = params_from_string s in
    let tbase = basetriangle pn pd qn qd rn rd bp in
    printf "Schwartz triangle: %i/%i %i/%i %i/%i\n%!" pn pd qn qd rn rd;
    printf "Starting point: %s\n\n\n%!" (match bp with
    | One -> "One"
    | Bissected -> "Bissected"
    | Incenter -> "Incenter"
    | Snub -> "Snub"
    | Custom x -> sprintf "Custom (%s)" (P.to_string x));
    printf "Base triangle: %s\n%!" (triangle_to_string tbase);
    let p = poly pn pd qn qd rn rd bp in
    printf "Number of vertices: %i\n%!" (Array.length p.vertices);
    printf "Number of edges: %i\n%!" (List.length p.edges);
    printf "Number of faces: %i\n%!" (List.length p.faces);
    print_newline ();
    print_newline ();
    printf "Vertices:\n%!";
    Array.iteri (fun i x -> printf "%i: %s\n%!" i (P.to_string x)) p.vertices;
    print_newline ();
    print_newline ();
    printf "Edges:\n%!";
    List.iter (fun (a, b) -> printf "%i <-> %i\n%!" a b) p.edges;
    print_newline ();
    print_newline ();
    printf "Faces:\n%!";
    List.iter (fun l -> List.iter (printf "%i ") l; print_newline ()) p.faces;
    print_newline ();
    print_newline ()
  in
  Arg.parse [] parse
    ("Usage: ./display.byte p q r with a bar (e.g. \"3 | 2 5\" or"
     ^ " \"3/2 3 2 |\")"
     ^ " or spherical coordinates between parentheses, omitting the radius"
     ^ " (e.g. \"p q r (0.4398, 3.087438)\"), the bar or coordinates"
     ^ " determines the starting point into the spherical triangle delimited"
     ^ " by the angles p, q and r.");
