(* File: vertices.ml

   Copyright (C) 2011-2015

     Fabian Pijcke <fabian.pijcke@gmail.com>
     Pierre Hauweele <antegallya@gmail.com>

   This library is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License version 3 or
   later as published by the Free Software Foundation, with the special
   exception on linking described in the file LICENSE.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file
   LICENSE for more details. *)

open Printf
open OUnit
open Common
open Wythoff

module P = Point
module M = Mirror

let symbols = [
  ("Tetrahedron", (3, 1, 2, 1, 3, 1, One), 4);
  ("Cube", (3, 1, 2, 1, 4, 1, One), 8);
  ("Octahedron", (4, 1, 2, 1, 3, 1, One), 6);
  ("Dodecahedron", (3, 1, 2, 1, 5, 1, One), 20);
  ("Isocahedron", (5, 1, 2, 1, 3, 1, One), 12);
  ("Truncated dodecahedron", (2, 1, 3, 1, 5, 1, Bissected), 60);
  ("Tetrahemihexahedron", (3, 2, 3, 1, 2, 1, Bissected), 6)
]

let count_vertices (_, (a, b, c, d, e, f, g), count) =
  let vertices = (poly a b c d e f g).Wythoff.vertices in
  assert_equal count (Array.length vertices)

let print_vertices (name, (a, b, c, d, e, f, g), _) =
  let vertices = (poly a b c d e f g).Wythoff.vertices in
  Array.iter print_string (Array.map (P.to_string ~after:"\n") vertices);
  print_newline ();
  print_newline ()

let tests = "Vertices tests" >::: [
  "Counting vertices" >:: (fun () ->
    List.iter count_vertices symbols
  );
  "Print vertices" >:: (fun () ->
    List.iter print_vertices symbols
  );
]
